let allLanguages = [
    { name: "English", key: "en" },
    { name: "العربية", key: "ar" },
    { name: "Deutsch", key: "de" },
    { name: "Español", key: "es" },
    { name: "Français", key: "fr" },
    { name: "Italiano", key: "it" },
    { name: "日本語", key: "ja" },
    { name: "ქართული", key: "ka" },
    { name: "Română", key: "ro" },
    { name: "Русский", key: "ru" },
    { name: "Slovensko", key: "si" },
    { name: "Українська", key: "uk" },
    { name: "Tiếng Việt", key: "vi" },
    { name: '中文', key: 'zh' },
    { name: "Portugues", key: "pt" }
]
export default allLanguages;