.cut-text { 
  text-overflow: ellipsis;
  overflow: hidden; 
  white-space: pre; /* preserve double white spaces (https://github.com/orthanc-server/orthanc-explorer-2/issues/30) */ 
  max-width: 0;
}

/* override CSS from datepicker to avoid the close button to overflow over the text */
.dp__input {
  padding-left: 4px !important;
  padding-right: 25px !important;
}

.dp__preset_ranges {
  width: 300px !important;
}

.dp__range_end, .dp__range_start, .dp__active_date {
  background: var(--dp-primary-color) !important;
}



/* for the StudyItem */
.label {
  margin-left: 2px;
  margin-left: 2px;
  background-color: var(--label-bg-color);
}

/* for the tag editor */
.tags-badge.bg-info  {
  background-color: var(--label-bg-color) !important;
}

.form-control::placeholder { /* Chrome, Firefox, Opera, Safari 10.1+ */
  color: var(--form-placeholder-color);
  opacity: 1; /* Firefox */
}

.job-card {
  margin-bottom: 2px;
}

.job-card a {
  /* color: black; */
}

.job-card-close {
  position: absolute;
  top: 5px;
  right: 5px;
}

.job-pct-pill {
  width: 50px;
  margin-right: 5px;
}

.jobs-header {
  text-align: left;
  padding-top: 2px;
  padding-bottom: 2px;
  line-height: 1.2rem;
}

.jobs-body {
  padding-top: 2px;
  padding-bottom: 2px;
  line-height: 1.6;
  text-align: left;
}

.jobs-body a {
  text-decoration: underline;
}


.btn-icon {
  line-height: 2rem;
  height: 2.5rem;
}

.btn-icon-small {
  height: 2rem;
  font-size: medium;
}