export var baseOe2Url = (!window.location.pathname.endsWith('/') ? '/' + window.location.pathname.split('/').slice(1, -1).join('/') + '/' : window.location.pathname);
export var orthancApiUrl = '../../'
export var oe2ApiUrl = '../api/';