/* Default CSS.  
This CSS is loaded after all other CSS files have been loaded 
therefore, it overrides all other definitions.  However, if you provide
a CustomCssPath, the custom file will be loaded after this one*/
:root {
    --bootstrap-theme: light;

    --study-even-bg-color: rgb(250, 250, 250);
    --study-odd-bg-color: rgb(240, 240, 240);
    --study-hover-color: rgb(60, 180, 255);
    --study-selected-color: rgb(235, 235, 235);
    --study-details-bg-color: var(--study-selected-color);
    --study-list-remote-bg-color: var(--study-hover-color);

    --series-even-bg-color: rgb(240, 240, 240);
    --series-odd-bg-color: rgb(235, 235, 235);
    --series-hover-color: var(--study-hover-color);
    --series-selected-color: rgb(225, 225, 225);
    --series-details-bg-color: var(--series-selected-color);

    --instance-odd-bg-color: rgb(230, 230, 230);
    --instance-even-bg-color: rgb(225, 225, 225);
    --instance-hover-color: var(--study-hover-color);
    --instance-selected-color: rgb(215, 215, 215);
    --instance-details-bg-color: var(--instance-selected-color);
    
    --study-table-header-bg-color: var(--study-even-bg-color);
    --study-table-filter-bg-color: rgb(220, 220, 220);
    --study-table-actions-bg-color: var(--study-table-filter-bg-color);

    --table-filters-is-searching-color: #fda90d86;
    --table-filters-is-not-searching-color: rgb(30, 150, 255);

    --details-top-margin: 25px;

    --nav-bar-width: 260px;

    --nav-side-color: rgb(255, 255, 255);
    --nav-side-bg-color: #2e353d;
    --nav-side-sub-bg-color: #181c20;
    --nav-side-selected-bg-color: #4f5b69;
    --nav-side-hover-bg-color: #3e454d;
    --nav-side-submenu-color: rgb(255, 255, 255);
    --nav-side-active-border-color: #d19b3d;

    --label-bg-color: rgb(10, 200, 240);

    --form-placeholder-color: #CCC;

    /* you may also customize global bootstrap variables */
    /* --bs-border-color: rgb(110, 160, 200); */
    /* --bs-info-rgb: 255, 26, 35; */
}

/* override the custom logo class */
/* .custom-logo {
    height: 150px;
} */


/* datepicker variables */
/* .dp__theme_dark {
    --dp-border-color: var(--bs-border-color);
} */

/* optional, you may e.g. customize bootstrap buttons or any other class that is declared globally */

/*  
.btn-primary {
    --bs-btn-color: white;
    --bs-btn-bg: red;
}  */
